package uloha1;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.stream.Collectors;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import processing.core.*;

public class Uloha1 extends PApplet {
    static public void main(String[] passedArgs) {PApplet.main("uloha1.Uloha1");}
    
    //K ovládání programu
    String adress = "";
    boolean UIControl;
    boolean dynamicResize;
    
    //Údaje o raketě a momentálnímu letu
    int flightID;
    int[] motors = new int[3];
    String[] rocketData;
    
    //K dynamickému zobrazování hrací plochy
    //float scaleFactor;

    //Obrázek rakety
    PShape modulePicture;
    
    @Override
    /**
     * Vestavěná funkce knihovny - zde probíhá základní setup.
     */
    public void settings() {
        String[] size = getStringInput("Zadejte velikost hrací plochy (normálně zadejte 200 200).", "Velikost plochy.").split(" ");

        //Změna velikosti okna simulace podle zadané velikosti.
        size(Integer.parseInt(size[0]), Integer.parseInt(size[1]));
    }
    
    /**
     * Základní setup - vestavěná funkce knihovny, spouští se při startu programu
     */
    @Override
    public void setup() {
        //Nahrání obrázku
        modulePicture = loadShape("C:\\final\\123\\Uloha1\\src\\uloha1\\rocketPicture.svg", "svg");
        
        //Zadání adresy (pouze jednou za celý běh programu) - default je http://localhost:8000/
        if ("".equals(adress)) adress = getStringInput("Zadejte adresu.", "Zadávání adresy.");
        
        //Odstartování letu a uložení ID
        try {
            rocketData = httpGet(adress+"start");
            flightID = Integer.parseInt(rocketData[0]);
        } catch (IOException e) {
            showDialog("Problém se serverem.");
            exit();
            return;
        }
        
        //Dotaz, zda bude let ovládán UI
        UIControl = (getYesNoInput("Chcete, aby raketu ovládal stroj (jen, pokud bude přímo nad přistávací plochou)?", "Advanced UI Request")==0);
    
        //Dotaz, zda dynamicky resizovat hrací plochu
        dynamicResize = (getYesNoInput("Chcete, aby se hrací plocha horizontálně vycentrovala na přistávací plochu?", "Dynamic Resize 4.96")==0);
    }
    
    /**
     * Vestavěná funkce knihovny - aktualizuje zobrazovací plochu
     */
    @Override
    public void draw() {
        //Bílé pozadí
        background(255);

        //Konvertuje souřadnice přistávací plochy, pozice a rychlosti rakety na arraye floatů
        //Tohle by bylo elegantnější přes objekty a classu, ale čas...
        Float[] podCoord = Arrays.stream(rocketData[1].split(" ")).map(Float::valueOf).toArray(Float[]::new);
        Float[] velocityData = Arrays.stream(rocketData[2].split(" ")).map(Float::valueOf).toArray(Float[]::new);
        Float[] landingCoord = Arrays.stream(rocketData[4].split(" ")).map(Float::valueOf).toArray(Float[]::new);

        try {
            //Pokud řídí UI, tak si upraví data motorů
            if (UIControl) {
                //Horizontální zpomalení
                if (velocityData[1]<-4.9) motors[0] = 1;
                else motors[0] = 0;
            }
            
            //httpGet s danými údaji motorů na server
            //Šlo by lehce udělat bez posílání argumentů, ale když to serveru nevadilo, tak mi to přišlo zbytečné
            rocketData = httpGet(adress+"stav?id="+flightID+"&nahoru="+motors[0]+"&doleva="+motors[1]+"&doprava="+motors[2]);
        } catch (IOException e) {
            showDialog("Problém se serverem.");
            exit();
            return;
        }
        
        //Test, zda není let u konce a případný konec simulace s možností restartu
        if ("Krach!".equals(rocketData[0])) {
            int answer = getYesNoInput("Ajaj. Bohužel jste havarovali. Chcete to zkusit znovu?", "Prohra. :-(");
            if (answer==0) setup();
            else exit();
            return;
        } else if ("Zdar!".equals(rocketData[0])) {
            int answer = getYesNoInput("Pěkná práce. Chcete letět ještě jednou?", "Výhra! :-)");
            if (answer==0) setup();
            else exit();
            return;
        }
            
        //Vypsání informací o stavu rakety na hrací pole
        fill(0);
        text("Rychlosti (Vx, Vy): "+rocketData[2], 10, 10);
        text("Palivo: "+rocketData[3], 10, 25);
        text("Motory: N"+motors[0]+", L"+motors[1]+", R"+motors[2], 10, 40);
        
        //Připraví plochu na vertikální otočení (viz volání scale funkce později)
        //Souřadnicový systém v téhle knihovně je z levého horního rohu [0, 0] a po y dolů se hodnoty zvětšují...
        translate(width/2, height);
        
        //Vycentruje herní plochu na pod
        if (dynamicResize) translate(-min(landingCoord[0], landingCoord[1])-max(landingCoord[0], landingCoord[1])/2, 0);
        
        //Horizontální otoření souřadnicového systému
        scale(1, -1);
        
        //Shape je SVG obrázek rakety, rect je přistávací plocha. Fill je pouze barva přistávací plochy (černobílá, 0-255)
        shape(modulePicture, podCoord[0], podCoord[1], 50, 50);
        fill(155);
        rect(landingCoord[0], 0, landingCoord[1], 10);
    }
    
    /**
     * Vestavěná funkce která se volá pokaždé, když je stisknuto tlačítko na klávesnici.
     */
    @Override
    public void keyPressed() {
        if (UIControl) return;
        switch(keyCode ) {
            case UP: motors[0] = 1;
                break;
            case LEFT: motors[1] = 1;
                break;
            case RIGHT: motors[2] = 1;
                break;
        }
    }
    
    /**
     * Vestavěná funkce která se volá pokaždé, když je uvolněno tlačítko na klávesnici.
     */
    @Override
    public void keyReleased() {
        if (UIControl) return;
        switch(keyCode) {
            case UP: motors[0] = 0;
                break;
            case LEFT: motors[1] = 0;
                break;
            case RIGHT: motors[2] = 0;
                break;
        }
    }
    
    /**
     * Požádá uživatele, aby zadal String.
     */
    public String getStringInput(String question, String title) {
        JFrame frame = new JFrame();
        String answer = JOptionPane.showInputDialog(frame, question, title, JOptionPane.DEFAULT_OPTION);
        frame.dispose();
        
        return answer;
    }
    
    /**
     * Pošle HTTP request na danou adresu
     * 
     * @param requestAdress Do jakého URL s jakými parametry půjde request?
     * @return Vrátí Sting array s výsledky
     */
    static String[] httpGet(String requestAdress) throws MalformedURLException, IOException {
        URL url = new URL(requestAdress);
        InputStream is = url.openStream();
        
        return new BufferedReader(new InputStreamReader(is)).lines().collect(Collectors.joining("\n")).split("\n");
    }
    
    /**
     * Požádá uživatele o an/ne odpověď na otázku.
     * 
     * @param question 0 je pravda, 1 je nepravda a cokoliv jiného je špatně!
     */
    public int getYesNoInput(String question, String title) {
        JFrame frame = new JFrame();
        int answer = JOptionPane.showConfirmDialog(frame, question, title, JOptionPane.YES_NO_OPTION);
        frame.dispose();
        
        return answer;
    }
    
    /**
     * Ukáže uživateli DialogBox.
     */
    static void showDialog(String message) {
        JFrame frame = new JFrame();
        JOptionPane.showMessageDialog(frame, message);
        frame.dispose();
    }
}
